#ifndef GUICOMMON_H
#define GUICOMMON_H

#include <iostream>
#include <QtGui>
#include "common.h"
#include "UIException.h"

// QComboBoxの itemDataの数字を比較して、一致するアイテムを選択する
void selectCombo(QComboBox *combo, int select);
void removeComboAll(QComboBox *combo);
// ログインユーザーの名前の取得
QString getUserName();

// 例外から、ダイアログを作成する
void exp_dialog(std::exception& exp, QWidget* parent=0);

// 以下のマクロは、thisを渡していので、マクロにせざるを得ない
#define DISP_DIALOG(exp) exp_dialog(exp, this) // 例外専用
#define DISP_INFO(message) QMessageBox::information(this, "harktool3", message)

void setWindowTitle(QWidget *widget);

template <typename T, typename T2> T safeCast(T2 type, bool exp=true) {
	T casted = dynamic_cast<T>(type);
	if (exp && !casted) {
		THROW_WARNING("failed to dynamic_cast");
	}
	return casted;
}

#ifdef DEBUG
	const QString TEST_MICFILE = "./DATA/MICARY-LocationFile";
	const QString TEST_SRCINF = "./DATA/ImpulseResponse-ListFile";
//	const QString TEST_SRCINF = "./DATA/TSP-ListFile";
#endif
	const QString TEST_FLT_TEMPLATE = "./DATA/flt/D#DEGREE#_ch*.flt";
	const QString TEST_TSP_TEMPLATE = "./DATA/tsp/D#DEGREE#.wav";

//! デバック用の一次元配列の調査クラス
/*!
 * 数値型Ｔのvectorに対して、平均値や最大最小のデータを取得できる
 */
template<typename T>
class CheckVector {
private:
	std::vector<T> &list;
public:
	T max;
	T min;
	double avg;
	double variance;
	int error;
	CheckVector(std::vector<T> &vec) :
		list(vec) {
		max = 0;
		min = 0;
		avg = 0;
		variance = 0;
		error = 0;
	}
	//! チェックするベクトルをセットする
	void setVector(std::vector<T> vec) {
		list = vec;
	}
	//! チェックの実行を開始する
	void execCheck() {
		if (!list.size())
			return;
		T total = 0;
		T pow2total = 0;

		// 初期値エラーチェック
		for (unsigned int i = 0; i < list.size(); i++) {
			if (isfinite(list[i])) {
				max = list[i];
				min = list[i];
				break;
			}
		}
		for (unsigned int i = 0; i < list.size(); i++) {
			if (isfinite(list[i])) {
				total += list[i];
				pow2total += list[i] * list[i];
				if (list[i] > max) {
					max = list[i];
				}
				if (list[i] < min) {
					min = list[i];
				}
			} else {
				error++;
			}
		}
		avg = total / (list.size() - error);
		variance = pow2total / (list.size() - error) - avg * avg;
		return;
	}
	//! 結果を出力する
	void print() {
		std::cout << "#CheckVector" << " size:" << list.size() << ", error:" << error << ",min:" << min << ",max:" << max
				<< ",avg:" << avg << ",variance:" << variance << std::endl;
	}
};

#endif
