#include "GUIMainWindow.h"
#include "ui_GUIMainWindow.h"

#include "GUINewFileSelect.h"
#include "GUIVerifyPanel.h"
#include "GUILocalizationTF.h"
#include "GUISeparationTF.h"
#include "GUISeparationMatrix.h"
#include "GUIMicaryLocationFile.h"
#include "GUINoiseLocationFile.h"
#include "GUICommon.h"
#include "GUIHelp.h"
#include "GUIConvertLocalizationTF.h"
#include "GUIConvertSeparation.h"
#include "GUIOldFormatSelect.h"

#include "UIApp.h"
#include "libharkio/libharkio.h"
using namespace harkio;

GUIMainWindow::GUIMainWindow(QWidget *parent) :
	QMainWindow(parent), ui(new Ui::GUIMainWindow) {
	ui->setupUi(this);

	::setWindowTitle(this);
	// サンプルのタブの全削除
	QWidget *center = centralWidget();
	delete center;

	uiUpdate(-1);
}

GUIMainWindow::~GUIMainWindow() {
IFDEBUG(DP("GUIMainWindow Destructor"));
	delete ui;
	UIRepository* repos = UIRepository::getRepository();
	repos->kill();
	repos->deleteTempFile();
}

QString GUIMainWindow::uniqueTabName(HT::UIApp::GROUPID id, QString name) {
	QString temp_name = name;
	int count=0;

	QTabWidget* tab2 = secondTabWidget(id);
	while (1) {
		bool find=false;
		for (int i=0; i < tab2->count(); i++) { // temp_nameを更新したら全タブを再検索
			if (tab2->tabText(i) == temp_name) {
				find=true;
			}
		}
		if (!find) {
			break;
		}
		count++;
		temp_name = QString("%1(%2)").arg(name).arg(count);
	}
	return temp_name;
}

GUITabDocument* GUIMainWindow::tabDocFactory(HT::UIApp::GROUPID id) {
	// parent の thisはコンテナに格納されるのでほとんど意味がない
	switch(id) {
	case HT::UIApp::GSS:
		return new GUISeparationTF(NULL, this);
		break;
	case HT::UIApp::MUSIC:
		return new GUILocalizationTF(NULL, this);
		break;
	case HT::UIApp::MIC:
		return new GUIMicaryLocationFile(NULL, this);
		break;
	case HT::UIApp::GSSMAT:
		return new GUISeparationMatrix(NULL, this);
		break;
	case HT::UIApp::NOISE:
		return new GUINoiseLocationFile(NULL, this);
		break;
	case HT::UIApp::TSP:
		return new GUITSPListFile(NULL, this);
		break;
	case HT::UIApp::IMPULSE:
		return new GUIImpulseResponseListFile(NULL, this);
		break;
	default:
		THROW_CRITICAL("Unknown GroupID for document's factory");
		break;
	}
}

QTabWidget* GUIMainWindow::addFileTypeWidget(HT::UIApp::GROUPID id) {

	QWidget *center = centralWidget();
	if (center == NULL) {
		QTabWidget *tab1 = new QTabWidget(this); // ファーストタブは１回だけ作成する
IFDEBUG(DP("FirstTab created =", tab1));
		setCentralWidget(tab1);
		tab1->setTabsClosable(true);
		tab1->setMovable(true);
		// ファーストタブの閉じるボタン
		connect(tab1, SIGNAL(tabCloseRequested(int)), this, SLOT(firstTabClose(int)));
	}
	// 新規にセカンドタブを作らないで現在のタブをかえす
	QTabWidget *tab2;
	tab2 = secondTabWidget(id);
	if (tab2) {
		return tab2;
	}
	// セカンドタブの新規作成(ファーストタブにウィジェットが追加された場合のみ)
	HT::UIApp *app = HT::UIApp::getInstance();
	QTabWidget *tab1 = qobject_cast<QTabWidget*>(centralWidget());
	tab2 = new QTabWidget(tab1);
IFDEBUG(DP("SecondTab created = ", tab2));
	tab2 ->setProperty("id", QVariant(id));
	tab1->addTab(tab2, app->groupName(id));
	tab2->setTabsClosable(true);
	tab2->setMovable(true);

	// セカンドタブの閉じるボタン
	connect(tab2, SIGNAL(tabCloseRequested(int)), this, SLOT(secondTabClose(int)));
	connect(tab2, SIGNAL(currentChanged(int)), this, SLOT(uiUpdate(int)));

	return tab2;
}

void GUIMainWindow::uiUpdate(int index) {
(void)index;
IFDEBUG(DP("## uiUpdate"));
	QTabWidget *tab = secondTabWidget(HT::UIApp::UNDEFINEDID, false);
	// FILE menu
	if (tab && tab->count()) {
		// ファイル
		ui->save_file->setDisabled(false);
		ui->save_file_as->setDisabled(false);
	} else {
		// ファイル
		ui->save_file->setDisabled(true);
		ui->save_file_as->setDisabled(true);
	}

	// Tool menu 変換
	if (tab) {

		GUITabDocument* guidoc = dynamic_cast<GUITabDocument*>(tab->currentWidget()); // tabがあってもページ(widget)が無いことは有る
		if (guidoc) {
			HT::UIDocument* uidoc = guidoc->getDoc();
			if (dynamic_cast<HT::MUSIC::UIDocument*>(uidoc)) {

				HT::MUSIC::UIDocument* uidoc2 = dynamic_cast<HT::MUSIC::UIDocument*>(uidoc);
				if (uidoc2->getUIData()->isHARKIO()) {
					ui->conversion->setDisabled(true);
				} else {
					ui->conversion->setDisabled(false);

					// ファイル(２度目)
					ui->save_file->setDisabled(true);
					ui->save_file_as->setDisabled(true);
				}
			} else if (dynamic_cast<HT::GSS::UIDocument*>(uidoc)) {
				HT::GSS::UIDocument* uidoc2 = dynamic_cast<HT::GSS::UIDocument*>(uidoc);
				if (uidoc2->getUIData()->isHARKIO()) {
					ui->conversion->setDisabled(true);
				} else {
					ui->conversion->setDisabled(false);

					// ファイル(２度目)
					ui->save_file->setDisabled(true);
					ui->save_file_as->setDisabled(true);
				}
			} else {
				ui->conversion->setDisabled(true);
			}
		}
	} else {
		ui->conversion->setDisabled(true);
	}

	// Tool menu　比較
	int count = 0;
	tab = secondTabWidget(HT::UIApp::GSSMAT, false);
	if (tab) {
		count += tab->count();
	}
	tab = secondTabWidget(HT::UIApp::GSS, false);
	if (tab) {
		count += tab->count();
	}
	tab = secondTabWidget(HT::UIApp::MUSIC, false);
	if (tab) {
		count += tab->count();
	}
	if (count >= 2) {
		ui->compare->setDisabled(false);
	} else {
		ui->compare->setDisabled(true);
	}
}

QTabWidget* GUIMainWindow::secondTabWidget(HT::UIApp::GROUPID id, bool exp) {

	QTabWidget *tab = qobject_cast<QTabWidget*>(centralWidget());
	if (!tab) {
		return tab;
	}
	QTabWidget *tab2 = NULL;

	if (id < 0) {
		tab2 = qobject_cast<QTabWidget *>(tab->currentWidget());
		if (exp && !tab2) {
			THROW_CRITICAL("Failed to get second tab widget");
		} else {
			return tab2;
		}
	} else {
		for (int i = 0; i < tab->count(); i++) {
			if (id == tab->widget(i)->property("id").toInt()) {
				tab2 = qobject_cast<QTabWidget*> (tab->widget(i));
			}
		}
	}
	return tab2;
}

void GUIMainWindow::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}

void GUIMainWindow::on_save_file_triggered()
{
	try {
		save(false);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUIMainWindow::on_save_file_as_triggered()
{
	try {
		save(true);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMainWindow::save(bool dialog) {

	QTabWidget *tab = secondTabWidget();
	GUITabDocument* guidoc = dynamic_cast<GUITabDocument*>(tab->currentWidget());

	if (!guidoc) {
		THROW_CRITICAL("Failed to get document in the tab widget");
	}
	guidoc->saveFile(dialog); // 各ドキュメントに対して保存の実行

	tab->setTabText(tab->currentIndex(), ""); // 重複タブを探すために前もって空にしておく

	HT::UIApp::GROUPID id = groupID(guidoc);
	QString unique = uniqueTabName(id, guidoc->getDoc()->getName().c_str());

	tab->setTabText(tab->currentIndex(), unique); // タブの名前更新
}

void GUIMainWindow::on_new_file_triggered()
{
	try {
//		GUINewFileSelect dialog(this);
		GUINewFileSelect* dialog = new GUINewFileSelect(this);
		connect(dialog, SIGNAL(createdData(GUITabDocument*)), this, SLOT(createDocument(GUITabDocument*)));
		dialog->show();

//		connect(&dialog, SIGNAL(createdData(GUITabDocument*)), this, SLOT(createDocument(GUITabDocument*)));
//		dialog.exec();
//		disconnect(&dialog, SIGNAL(createdData(GUITabDocument*)), this, SLOT(createDocument(GUITabDocument*)));
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMainWindow::createDocument(GUITabDocument *guidoc) {
IFDEBUG(DP("createDocument"));

	try {
		HT::UIApp::GROUPID id = groupID(guidoc);

		// タブの作成
		QTabWidget *tab2 = addFileTypeWidget(id);

		QTabWidget *tab1 = firstTabWidget();
		tab1->setCurrentWidget(tab2);

		tab2->addTab(guidoc, guidoc->getDoc()->getName().c_str());
		tab2->setCurrentWidget(guidoc);

		tab2->setTabText(tab2->currentIndex(), ""); // 重複タブを探すために前もって空にしておく
		QString unique = uniqueTabName(id, guidoc->getDoc()->getName().c_str());
		tab2->setTabText(tab2->currentIndex(), unique); // タブの名前更新

		// タブの名前の更新
		tab2->setTabText(tab2->currentIndex(), unique);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMainWindow::openFileByCommandLine(QString fname) {

	try {
		openFile(fname);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMainWindow::openFile(QString fname) {

		// HARKIO バイナリデータ
		{
			HARKIO_RET ret = harkio::HARK::checkIdentifier(fname.toStdString());
			if (ret == harkio::HARKIO_TRUE) {
				HARK hark;
				hark.readFile(fname.toStdString());

				GUITabDocument* guidoc;
				//				QTabWidget *tab1 = firstTabWidget();
				//				QTabWidget *tab2;

				if (hark.getData()->getChunk<M2PG> ()) {
					guidoc = tabDocFactory(HT::UIApp::MUSIC);
				} else if (hark.getData()->getChunk<GTF> ()) {
					guidoc = tabDocFactory(HT::UIApp::GSS);
				} else if (hark.getData()->getChunk<SM> ()) {
					guidoc = tabDocFactory(HT::UIApp::GSSMAT);
				} else {
					THROW_CRITICAL(tr("Can't process file format for HARKIO"));
				}
				guidoc->openFile(fname);
				createDocument(guidoc);
				HT::UIDocument::setLastPath(fname.toStdString());
				goto END;
			}
		}
		// HARKIO テキストデータ
		{
			harkio::TextData text;
			int type = text.checkFileIdentifier(fname.toStdString());
			HT::UIApp *app = HT::UIApp::getInstance();
			HT::UIApp::GROUPID id = app->convIDHarkToolTextToDocument(type);
			if (id >= 0) {
				GUITabDocument* guidoc = tabDocFactory(id);
				guidoc->openFile(fname);
				createDocument(guidoc);
				HT::UIDocument::setLastPath(fname.toStdString());
				goto END;
			}
		}
		// それ以外は旧ファイル形式など
		{
			GUIOldFormatSelect dialog(this);
			dialog.fname = fname;
			dialog.exec();
		}
		END: ;
}
void GUIMainWindow::on_open_file_triggered()
{
	try {
		QString fname = QFileDialog::getOpenFileName(this, "", HT::UIDocument::getLastPath().c_str());
		if (fname == "")
			return;

		openFile(fname);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMainWindow::on_compare_triggered()
{
	try {
			GUIVerifyPanel dialog(this);
			dialog.exec();
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMainWindow::on_about_triggered() {

	try {
		GUIHelp dialog(this);
		dialog.exec();
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUIMainWindow::on_conversion_triggered() {

	try {
		QTabWidget *tab = secondTabWidget();
		GUITabDocument* guidoc = dynamic_cast<GUITabDocument*>(tab->currentWidget());
		HT::UIDocument* uidoc = guidoc->getDoc();

		if (dynamic_cast<HT::MUSIC::UIDocument*>(uidoc)) {
			GUIConvertLocalizationTF dialog(this);
			dialog.exec();
		} else if (dynamic_cast<HT::GSS::UIDocument*>(uidoc)) {
			GUIConvertSeparation dialog(this);
			dialog.exec();
		} else {
			THROW_CRITICAL("Unknown document for Conversion");
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMainWindow::firstTabClose(int index) {

	try {
		QTabWidget* tab = firstTabWidget();
		// widgetを消すとremoveTabと同じ動作になる模様、ヘルプにはないので安全な方法を使用する
		tab->widget(index)->deleteLater();
		tab->removeTab(index);
		if (!tab->count()) {
			uiUpdate(-1);
		}

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}

}
void GUIMainWindow::secondTabClose(int index) {
	try {
		QTabWidget* tab2 = secondTabWidget();
		// widgetを消すとremoveTabと同じ動作になる模様、ヘルプにはないので安全な方法を使用する
		tab2->widget(index)->deleteLater();
		tab2->removeTab(index);
		if (!tab2->count()) {
			uiUpdate(-1);
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}
