#ifndef GUIMainWindow_H
#define GUIMainWindow_H

#include <QMainWindow>
#include "common.h"

#include "UIDatData.h"
#include "UIDatDocument.h"
#include "UIGraph.h"
#include <QtGui>

#include "GUILocalizationTF.h"
#include "GUISeparationTF.h"
#include "GUISeparationMatrix.h"
#include "GUIMicaryLocationFile.h"
#include "GUINoiseLocationFile.h"
#include "GUIImpulseResponseListFile.h"
#include "GUITSPListFile.h"

namespace Ui {
class GUIMainWindow;
}

class GUIMainWindow: public QMainWindow {
Q_OBJECT
private: //Singleton Pattern
	GUIMainWindow(QWidget *parent = 0);
public:
	~GUIMainWindow();
	static GUIMainWindow *getMainWindow() { // 依存関係削減の為に、余程の事が無ければ使用しない事!
		static GUIMainWindow *window=NULL;
		if (!window) {
			window = new GUIMainWindow;
		}
		return window;
	}
 	HT::UIApp::GROUPID groupID(GUITabDocument* doc) {
 		if (dynamic_cast<GUISeparationMatrix*> (doc)) {
 			return HT::UIApp::GSSMAT;
 		} else {
 			HT::UIApp *app = HT::UIApp::getInstance();
 			return app->groupID(doc->getDoc());
 		}
 	}
	// タブ内に表示するウィジェットのファクトリ
	GUITabDocument* tabDocFactory(HT::UIApp::GROUPID id);
	// カレントタブの種別を取得する
	int currentFileType() {
		QTabWidget *tab = qobject_cast<QTabWidget*>(centralWidget());
		return tab->currentWidget()->property("id").toInt();
	}

	// タブの操作関連関数
	QTabWidget* firstTabWidget() {
		QTabWidget *tab = qobject_cast<QTabWidget*>(centralWidget());
		if (!tab) {
			THROW_CRITICAL("failed to get first tab widget");
		}
		return tab;
	}
	QTabWidget* secondTabWidget(HT::UIApp::GROUPID id=HT::UIApp::UNDEFINEDID, bool exp=false); // idがマイナスの時は、カレントタブを取得する
	QTabWidget* addFileTypeWidget(HT::UIApp::GROUPID type);
	QString uniqueTabName(HT::UIApp::GROUPID id, QString name="Untitled");

	void changeEvent(QEvent *e);
	void save(bool dialog);
	void openFile(QString fname);
	void openFileByCommandLine(QString fname);
private:
	Ui::GUIMainWindow *ui;
public slots:
	void on_new_file_triggered();
public slots:
	void uiUpdate(int index=0); // dummy argument
private slots:
	void on_save_file_triggered();
	void on_save_file_as_triggered();
	void on_open_file_triggered();
	void firstTabClose(int);
	void secondTabClose(int);
	void on_conversion_triggered();
	void on_compare_triggered();
	void on_about_triggered();
public slots:
	void createDocument(GUITabDocument* guidoc);
};

#endif // GUIMainwindow_H
