#include "GUIMicaryLocationFile.h"
#include "ui_GUIMicaryLocationFile.h"

#include "GUICommon.h"
#include "UIDocument.h"
#include "UIGraph.h"
#include "GUIMicaryLocationFileTemplateNew.h"
#include "libharkio/libharkio.h"
using namespace harkio;

GUIMicaryLocationFile::GUIMicaryLocationFile(HT::MIC::UIDocument* doc, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::GUIMicaryLocationFile)
{
	ui->setupUi(this);
	::setWindowTitle(this);
	if (doc) {
		this->doc = doc;
	} else {
		this->doc = new HT::MIC::UIDocument();
	}
}

GUIMicaryLocationFile::~GUIMicaryLocationFile()
{
IFDEBUG(DP("GUIMicaryLocationFile destractor"));
	delete ui;
	delete doc;
}

void GUIMicaryLocationFile::templateNew() {

	try {
		GUIMicaryLocationFileTemplateNew dialog(this);
		connect(&dialog, SIGNAL(createdTemplate(QString)), this, SLOT(dispTemplate(QString)) );
		dialog.exec();
		disconnect(&dialog, SIGNAL(createdTemplate(QString)), this, SLOT(dispTemplate(QString)) );
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}


void GUIMicaryLocationFile::openFile(QString file) {

	std::ifstream ifs(file.toStdString().c_str());
	if(ifs.fail())
	{
		THROW_WARNING(QString(tr("Can't open file:%1")).arg(file));
	}
	std::stringstream buf;
	buf << ifs.rdbuf();
	dispTemplate(buf.str().c_str());

	doc->setFileName(file.toStdString());
}

void GUIMicaryLocationFile::dispTemplate(QString text) {
	try {
		ui->text->setPlainText(text);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMicaryLocationFile::dispGraph() {

	try {

		QString text = ui->text->toPlainText();
		// ファイルの取得
		harkio::MICARYLocationFile file;
		file.readText(text.toStdString());

		// データの取得
		std::vector<harkio::UIMic<> > *mic_lists = &(file.mic_lists);
		std::vector<double> xyz(3);
		std::vector<std::vector<double> > lists; // マイク位置

		for (uint i=0 ; i < mic_lists->size(); i++) {
			xyz[0] = (*mic_lists)[i].x;
			xyz[1] = (*mic_lists)[i].y;
			xyz[2] = (*mic_lists)[i].z;
			lists.push_back(xyz);
		}
		// プロット
		ScatterPlot<double> graph(&lists);
		graph.setTitle("MICARY-LocationFile");
		graph.setStyle(ScatterPlot<>::PlotStyle::LINESPOINTS);
		graph.plot(harktoolDir());

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUIMicaryLocationFile::saveFile(bool dialog) {

	try {
		QString fname = doc->getFileName().c_str();
		bool saved=false;
		if (!fname.isEmpty()) {
			saved = true;
		}
		if (dialog || !saved) {
			fname = QFileDialog::getSaveFileName(this, "", doc->getLastPath(fname.toStdString()).c_str());
			if (fname.isEmpty()) {
				return;
			}
		}
		
		QFile file(fname);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
			THROW_WARNING(QString(tr("Can't open file:%1")).arg(fname));
			return;
		}
		doc->setFileName(fname.toStdString());
		HT::UIDocument::setLastPath(fname.toStdString());
		
		// 書き込み
		QTextStream out(&file);
		out << ui->text->toPlainText();
		if (!dialog && saved) {
			DISP_INFO(tr("File Overwritten", "上書き保存しました"));
		}

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

QString GUIMicaryLocationFile::text() {
	return ui->text->toPlainText();
}

void GUIMicaryLocationFile::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
