#include "GUISeparationTF.h"
#include "ui_GUISeparationTF.h"

#include "GUICommon.h"
#include "GUIPlotParameterPanel.h"
#include "UITFDocument.h"
#include "libharkio/libharkio.h"
using namespace harkio;

GUISeparationTF::GUISeparationTF(HT::GSS::UIDocument* doc, QWidget *parent) :
	QWidget(parent), ui(new Ui::GUISeparationTF) {

	ui->setupUi(this);

	if (doc) {
		this->doc = doc;
		disp();
	} else {
		this->doc = new HT::GSS::UIDocument();
	}
}

GUISeparationTF::~GUISeparationTF() {
	delete ui;
	delete doc;
}

void GUISeparationTF::dispPanel(QTreeWidgetItem *item, int column) {
(void) column;
	try {
		if (item == item_tf) {
			GUIPlotParameterPanel *panel = new GUIPlotParameterPanel(doc, this);
			panel->exec();
		} else if (item == item_src) {

			HT::GSS::UIData *data = doc->getUIData();
			std::vector<std::vector<double> > lists;

			if (data->getHARKIO()) {
				GTF *gtf = data->getHARKIO()->getData()->getChunk<GTF>();
				GTF::SRC_LIST_TYPE *src = gtf->getSrc();
				for (uint i=0; i < (*src).size(); i++) {
						std::vector<double> xyz(3);
						xyz[0] = (*src)[i].x;
						xyz[1] = (*src)[i].y;
						xyz[2] = (*src)[i].z;
						lists.push_back(xyz);
				}
			} else {
				HT::Transform transform;
				lists = transform.OldSrcToGNUPlotVector(data->getSrc());
			}
			ScatterPlot<double> graph(&lists);
			graph.setTitle("Source Position");
			graph.setStyle(ScatterPlot<>::PlotStyle::LINESPOINTS);
			graph.plot(harktoolDir());
		} else if (item == item_mic) {
				HT::GSS::UIData *data = doc->getUIData();
				std::vector<std::vector<double> > lists;
				if (data->getHARKIO()) {
					FMT *fmt = data->getHARKIO()->getData()->getChunk<FMT>();
					FMT::MIC_LIST_TYPE *mic = fmt->getMic();
					for (uint i=0; i < (*mic).size(); i++) {
							std::vector<double> xyz(3);
							xyz[0] = (*mic)[i].x;
							xyz[1] = (*mic)[i].y;
							xyz[2] = (*mic)[i].z;
							lists.push_back(xyz);
					}
				} else {
					HT::Transform transform;
					lists = transform.OldMicToGNUPlotVector(data->getMic());
				}
				ScatterPlot<double> graph(&lists);
				graph.setTitle("Microphone Position");
				graph.setStyle(ScatterPlot<>::PlotStyle::LINESPOINTS);
				graph.plot(harktoolDir());
		} else {
			// ツリーのすべての場所でクリックイベントが発生する
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUISeparationTF::disp() {

	HT::GSS::UIData *data = doc->getUIData();
	if (data->getHARKIO()) {
		dispHARKIOData();
	} else {
		dispOldData();
	}
}

void GUISeparationTF::dispHARKIOData() {

	HT::GSS::UIData *data = doc->getUIData();
	QTreeWidget *tree = ui->treeWidget;
	tree->clear();

	if (!data->getHARKIO()) {
		return;
	}

	QStringList line;
	QTreeWidgetItem * item;
	QTreeWidgetItem * item2;

	connect(tree, SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int)), this, SLOT(dispPanel(QTreeWidgetItem *, int)) );
	tree->header()->setResizeMode(0, QHeaderView::ResizeToContents);
	//	line << "フォーマット名" << "値";
	//	tree->setHeaderLabels(line);
	//	line.clear();

	//FMT
	FMT *fmt = data->getHARKIO()->getData()->getChunk<harkio::FMT> ();

	line << " fmt";
	item = new QTreeWidgetItem(tree, line);
	item->setExpanded(true);
	tree->addTopLevelItem(item);
	line.clear();

	line << tr("Version", "バージョン") << QString("%1").arg(fmt->getVersionStr().c_str());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Format ID","フォーマットID") << QString("%1").arg(fmt->getFormatId());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Num of Mics","マイク数") << QString("%1").arg(fmt->getNMic());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("FFT Length[sample]","FFT長[サンプル]") << QString("%1").arg(fmt->getNFFT());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("IR Length[sample]","インパルスレスポンス長[サンプル]") << QString("%1").arg(fmt->getNIR());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Sampling Rate[Hz]","サンプリング周波数[Hz]") << QString("%1").arg(fmt->getNFS());
	;
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Mic Position","マイク位置") << tr("Plot", "プロットする");
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	item_mic = item2; // ダブルクリックイベントの為に保持
	line.clear();

	//GTF
	GTF *gtf = data->getHARKIO()->getData()->getChunk<GTF> ();

	line << "GTF";
	item = new QTreeWidgetItem(tree, line);
	item->setExpanded(true);
	tree->addTopLevelItem(item);
	line.clear();

	line << tr("Version","バージョン") << QString("%1").arg(gtf->getVersionStr().c_str());
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	line.clear();

	line << tr("Num of Sources","音源位置数") << QString("%1").arg(gtf->getNSrc());
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	line.clear();

	line << tr("Source Location","音源位置の座標") << tr("Plot", "プロットする");
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	item_src = item2; // ダブルクリックイベントの為に保持
	line.clear();
	/*
	 line << "周波数ライン数" << QString("%1").arg(gtf->getNFI());
	 item2 = new QTreeWidgetItem(item, line);
	 tree->addTopLevelItem(item2);
	 line.clear();

	 line << "インデックスから周波数ビンへのマップ" << "プロットする";
	 item2 = new QTreeWidgetItem(item, line);
	 tree->addTopLevelItem(item2);
	 line.clear();
	 */
	line << tr("Transfer Function","伝達関数") << tr("Plot", "プロットする");
	item2 = new QTreeWidgetItem(item, line);
	item_tf = item2; // ダブルクリックイベントの為に保持
	tree->addTopLevelItem(item2);
	line.clear();

	// SPEC
	SPEC *spec = data->getHARKIO()->getData()->getChunk<SPEC> ();

	line << "SPEC";
	item = new QTreeWidgetItem(tree, line);
	item->setExpanded(true);
	tree->addTopLevelItem(item);
	line.clear();

	line << tr("Date","作成日") << QString("%1").arg(spec->getDate());
	item2 = new QTreeWidgetItem(item, line);
	item2->setFlags(item2->flags() | Qt::ItemIsEditable);
	tree->addTopLevelItem(item2);
	spec_date = item2;
	line.clear();

	line << tr("Place","場所") << QString("%1").arg(spec->getLocation());
	item2 = new QTreeWidgetItem(item, line);
	item2->setFlags(item2->flags() | Qt::ItemIsEditable);
	tree->addTopLevelItem(item2);
	spec_location = item2;
	line.clear();

	line << tr("Robot","ロボット") << QString("%1").arg(spec->getRobot());
	item2 = new QTreeWidgetItem(item, line);
	item2->setFlags(item2->flags() | Qt::ItemIsEditable);
	tree->addTopLevelItem(item2);
	spec_robot = item2;
	line.clear();

	line << tr("Array Type","アレイのタイプ") << QString("%1").arg(spec->getArrayType());
	item2 = new QTreeWidgetItem(item, line);
	item2->setFlags(item2->flags() | Qt::ItemIsEditable);
	tree->addTopLevelItem(item2);
	spec_array_type = item2;
	line.clear();

	line << tr("Creator","作成者") << QString("%1").arg(spec->getCreator());
	item2 = new QTreeWidgetItem(item, line);
	item2->setFlags(item2->flags() | Qt::ItemIsEditable);
	tree->addTopLevelItem(item2);
	spec_creator = item2;
	line.clear();

	line << tr("Notes","備忘録") << QString("%1").arg(spec->getMemo());
	item2 = new QTreeWidgetItem(item, line);
	item2->setFlags(item2->flags() | Qt::ItemIsEditable);
	tree->addTopLevelItem(item2);
	spec_memo = item2;
	line.clear();

	return;
}

void GUISeparationTF::dispOldData() {

	HT::GSS::UIData *data = doc->getUIData();
	QTreeWidget *tree = ui->treeWidget;
	tree->clear();

	if (!data->getMatrix() || !data->getSrc()) {
		return;
	}

	QStringList line;
	QTreeWidgetItem *item;
	QTreeWidgetItem *item2;

	connect(tree, SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int)), this, SLOT(dispPanel(QTreeWidgetItem *, int)) );
	tree->header()->setResizeMode(0, QHeaderView::ResizeToContents);
//	line << "フォーマット名" << "値";
//	tree->setHeaderLabels(line);
//	line.clear();

	line << tr("Old Formated Data", "旧形式データファイル");
	item = new QTreeWidgetItem(tree, line);
	item->setExpanded(true);
	tree->addTopLevelItem(item);
	line.clear();

	line << tr("Version", "バージョン") << QString("%1").arg(data->getVersion());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("File Type", "ファイルタイプ") << QString("%1").arg(data->getFileType());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Sampling Rate[Hz]", "サンプリング周波数[Hz]") << QString("%1").arg(data->getNFS());;
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("FFT Length[sample]", "FFT長(サンプル)") << QString("%1").arg(data->getNFFT());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Num of Freq Bins", "周波数ライン数") << QString("%1").arg(data->getNFreq());
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	line.clear();

	line << tr("Enable Mesh", "メッシュ使用") << QString("%1").arg(data->getUseMesh());
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	line.clear();

	line << tr("Num of Sources", "ソース数") << QString("%1").arg(data->getNSrc());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Num of Mics", "マイク数") << QString("%1").arg(data->getNMic());
	item2 = new QTreeWidgetItem(item, line);
	line.clear();

	line << tr("Mic Position", "マイク位置") << tr("Plot", "プロットする");
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	item_mic = item2; // ダブルクリックイベントの為に保持
	line.clear();

	line << tr("Source Location", "音源座標") << tr("Plot", "プロットする");
	item2 = new QTreeWidgetItem(item, line);
	tree->addTopLevelItem(item2);
	item_src = item2; // ダブルクリックイベントの為に保持
	line.clear();

	line << tr("Transfer Function", "伝達関数") << tr("Plot", "プロットする");
	item2 = new QTreeWidgetItem(item, line);
	item_tf = item2; // ダブルクリックイベントの為に保持
	tree->addTopLevelItem(item2);
	line.clear();

	return;
}

void GUISeparationTF::openFile(QString file) {

	doc->loadFile(file.toStdString());
	// 内容の表示
	disp();
	doc->setFileName(file.toStdString());
}

void GUISeparationTF::saveFile(bool dialog) {

	QString fname = doc->getFileName().c_str();

	if (dialog || fname == "") {
		fname = QFileDialog::getSaveFileName(this, doc->getLastPath(fname.toStdString()).c_str());
		if (fname == "") {
			return;
		}
	}
	QFile file(fname);
	if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
		THROW_WARNING(QString(tr("Can't open file:%1")).arg(fname));
		return;
	}
	doc->setFileName(fname.toStdString());
	HT::UIDocument::setLastPath(fname.toStdString());

	HT::GSS::UIData *data = doc->getUIData();

	//SPEC データ更新
	harkio::HARK *hark = data->getHARKIO();
	harkio::SPEC *spec = hark->getData()->getChunk<SPEC>();

	spec->setDate(QTOC(spec_date->text(1)));
	spec->setLocation(QTOC(spec_location->text(1)));
	spec->setRobot(QTOC(spec_robot->text(1)));
	spec->setArrayType(QTOC(spec_array_type->text(1)));
	spec->setCreator(QTOC(spec_creator->text(1)));
	spec->setMemo(QTOC(spec_memo->text(1)));

	// 書き込み
	data->getHARKIO()->writeFile(fname.toStdString());
	fname = fullPathToFile(fname.toStdString()).c_str();

}

void GUISeparationTF::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
