#include "GUISeparationTFNew.h"
#include "ui_GUISeparationTFNew.h"

#include "UIDatData.h"
#include "GUICommon.h"
#include "GUISeparationTF.h"
#include "UIApp.h"
#include "UIGraph.h"
#include "UIException.h"
#include "UIDatDocument.h"
#include "GUIMainWindow.h"

#include "libharkio/libharkio.h"
using namespace harkio;

// 2012-03-07 (s.noguchi) MICARY,Impulseファイルの保存とリストア
const string MIC_FILE_PATH = "micfile_path.txt";
const string IMP_FILE_PATH = "impfile_path.txt";


GUISeparationTFNew::GUISeparationTFNew(QWidget *parent) :
	QDialog(parent),
	ui(new Ui::GUISeparationTFNew)
{
	::setWindowTitle(this);
	ui->setupUi(this);
	::setWindowTitle(this);

	QString micfile;
	QString srcinf;
	QDir dir;

#ifdef DEBUG
	micfile = TEST_MICFILE;
	srcinf = TEST_SRCINF;
#else
	// 2012-03-07 (s.noguchi) MICARY,Impulseファイルの保存とリストア
	string file_location, file_location_result;

	file_location.clear();
	file_location_result = getFileLocation(MIC_FILE_PATH, file_location);
	if (file_location_result.size() == 0)
		micfile = QString("%1/YOUR_PATH/MICARY-LocationFile").arg(dir.homePath());
	else
		micfile = QString(file_location_result.c_str());

	file_location.clear();
	file_location_result = getFileLocation(IMP_FILE_PATH, file_location);
	if (file_location_result.size() == 0)
		srcinf = QString("%1/YOUR_PATH/ImpulseResponse-ListFile or TSP-ListFile").arg(dir.homePath());
	else
		srcinf = QString(file_location_result.c_str());
#endif

	connect(ui->mic_file, SIGNAL(activated(int)), this, SLOT(changeComboBox(int)));
	connect(ui->srcinf_file, SIGNAL(activated(int)), this, SLOT(changeComboBox(int)));
	ui->mic_file->addItem(micfile);
	ui->srcinf_file->addItem(srcinf);

	signal_data_clicked();
}

GUISeparationTFNew::~GUISeparationTFNew()
{
	delete ui;
}

void GUISeparationTFNew::changeComboBox(int index) {
	try {
		QComboBox* combo = qobject_cast<QComboBox*>(sender());
		if (index) {
			combo->lineEdit()->setReadOnly(true);
		} else {
			combo->lineEdit()->setReadOnly(false);
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUISeparationTFNew::updateComboBox() {
	for (int i=0; i < impulse_files.size(); i++) {
		ui->srcinf_file->addItem(impulse_files[i].first, impulse_files[i].second);
	}
	for (int i=0; i < tsp_files.size(); i++) {
		ui->srcinf_file->addItem(tsp_files[i].first, tsp_files[i].second);
	}
	for (int i=0; i < mic_files.size(); i++) {
		ui->mic_file->addItem(mic_files[i].first, mic_files[i].second);
	}
}

void GUISeparationTFNew::mic_location_clicked() {
	QString fname = QFileDialog::getOpenFileName(this, "MICARY-LocationFile", HT::UIDocument::getLastPath(ui->mic_file->lineEdit()->text().toStdString()).c_str());
	if (fname != "") {
		ui->mic_file->lineEdit()->setText(fname);
		HT::UIDocument::setLastPath(fname.toStdString());
	}
}

void GUISeparationTFNew::src_location_clicked() {
	QString fname = QFileDialog::getOpenFileName(this, "ImpulseResponse-ListFile", HT::UIDocument::getLastPath(ui->srcinf_file->lineEdit()->text().toStdString()).c_str());
	if (fname != "") {
		ui->srcinf_file->lineEdit()->setText(fname);
		HT::UIDocument::setLastPath(fname.toStdString());
	}
}

void GUISeparationTFNew::changeEvent(QEvent *e) {
	QDialog::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}

void GUISeparationTFNew::ok_clicked() {
	HT::GSS::UIDocument *doc = NULL;
	GUISeparationTF* guidoc = NULL;
	HT::GSS::UIData *data = NULL;

	try {
		HT::GSS::CreateTFParam param; // すべてのパラメータを格納

		// 数字パラメータ格納
		param.NFFT = ui->fft->value();
		param.NFS = ui->fs->value();
		param.IMPULSE_RESPONSE_LEN = ui->impulse_response_length->value();

		HT::UIApp *app = HT::UIApp::getInstance();

		{// エラーチェック
			std::string fname;
			QString imp_fname;
			harkio::TextData text;
			int type;


			// MICARY-LocationFile のチェック
			imp_fname = ui->mic_file->itemData(ui->mic_file->currentIndex()).toString();
			if (imp_fname != "") {
				fname = imp_fname.toStdString();
			} else {
				fname = fname = ui->mic_file->lineEdit()->text().toStdString();
			}
			if (fname == "") {
				THROW_WARNING(tr("Input Micary-LcationFile", "Micary-LcationFileを入力してください"));
			}
			type = text.checkFileIdentifier(fname);
			if (app->convIDHarkToolTextToDocument(type) != HT::UIApp::MIC) {
				THROW_WARNING(tr("Unknown format:Micary-LcationFile", "Micary-LcationFileが不明なフォーマットです"));
			}
			param.mic_file = fname; // 入力マイクファイル

			saveFileLocation(MIC_FILE_PATH, fname.c_str());


			// ImpulseResponse-ListFIle or TSP-ListFile のチェック
			imp_fname = ui->srcinf_file->itemData(ui->srcinf_file->currentIndex()).toString();
			if (imp_fname != "") {
				fname = imp_fname.toStdString();
			} else {
				fname = ui->srcinf_file->lineEdit()->text().toStdString();
			}
			if (fname == "") {
				if (ui->impulseresponse->isChecked()) {
					THROW_WARNING(tr("Input ImpulseResponse-ListFile", "ImpulseResponse-ListFileを入力してください"));
				} else {
					THROW_WARNING(tr("Input TSP-ListFile", "TSP-ListFileを入力してください"));
				}
			}
			type = text.checkFileIdentifier(fname);
			if (ui->impulseresponse->isChecked()) {
				if (app->convIDHarkToolTextToDocument(type) != HT::UIApp::IMPULSE) {
					THROW_WARNING(tr("Unknown format:ImpulseResponse-ListFile", "ImpulseResponse-ListFileが不明なフォーマットです"));
				}
			} else if (ui->tsp->isChecked()) {
				if (app->convIDHarkToolTextToDocument(type) != HT::UIApp::TSP) {
					THROW_WARNING(tr("Unknown format:TSP-ListFile", "TSP-ListFileが不明なフォーマットです"));
				}
			}
			param.src_file = fname; // 入力ソースファイル

			saveFileLocation(IMP_FILE_PATH, fname.c_str());

		} //エラーチェック

		doc = new HT::GSS::UIDocument();
		data = doc->newTFData(doc);

		// 2012-02-22 (s.noguchi) 正規化方法選択
		int normalization;
		if (ui->ch->isChecked() ) {
			normalization = HT::GSS::UIData::CH;			// チャネル 正規化
		} else if (ui->ch_frq->isChecked() ) {
			normalization = HT::GSS::UIData::CH_FRQ;		// チャネル+周波数
		} else if (ui->ch_frq_dir->isChecked() ) {
			normalization = HT::GSS::UIData::CH_FRQ_DIR;	// チャネル+周波数+方向
		} else { // None 
			normalization = HT::GSS::UIData::NONE;			// 何もしない
		}
		data->setNormalization(normalization);

		// ダイアログ更新イベントの取得
		connect(data, SIGNAL(createdTSP(int,int,HT::UIData*)), this, SLOT(createdTSP(int,int, HT::UIData*)));
		// データ作成 ここが時間がかかる
		data->createTFData(param);
		connect(data, SIGNAL(createdTSP(int,int,HT::UIData*)), this, SLOT(createdTSP(int,int,HT::UIData*)));
		// SPECデータの上書き
		SPEC* spec = data->getHARKIO()->getData()->getChunk<SPEC>();
		spec->setDate(QDateTime::currentDateTime().toString("yyyy/MM/dd").toStdString().c_str());
		spec->setCreator(getUserName().toStdString().c_str());
		guidoc = new GUISeparationTF(doc);
		// ウィンドウに通知
		emit createdData(guidoc);
		done(-1);

	} catch (std::exception& exp) {
		if (guidoc) {
			delete guidoc;
		} else if(doc) {
			delete doc;
		} else if (data) {
			delete data;
		}
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUISeparationTFNew::signal_data_clicked() {
	try {
		if (ui->impulseresponse->isChecked()) {
			ui->srcinf_label->setText("ImpulseResponse-ListFile");
		} else if (ui->tsp->isChecked()) {
			ui->srcinf_label->setText("TSP-ListFile");
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUISeparationTFNew::createdTSP(int current, int total, HT::UIData *data) {
	static QProgressDialog *progress=NULL;
	if (current < 0) {
		delete progress;
		progress = NULL;
		return;
	}
	if (progress == NULL) {
		// プログレスバーダイアログの作成
		progress = new QProgressDialog(this);
		progress->setWindowTitle(tr("Generating impulse response...", "インパルス応答の作成中..."));
		progress->setRange(0, total);
		progress->setModal(true);
	}
	if (progress->wasCanceled()) {
		delete progress;
		progress = NULL;
		// シグナルの送信元にキャンセルフラグを立てる
		safeCast<HT::GSS::UIData*>(data)->setTSPCanceled(true);
		return;
	}
	progress->setValue(current);
}

/*
 * 2012-03-07 (s.noguchi) MICARY,Impulseファイルの保存とリストア
 */
string GUISeparationTFNew::getFileLocation(string fname, string location) {
	QDir dir;
	QString path = QString("%1/%2/%3").arg(dir.homePath(), TEMPORARY_PATH.c_str(), fname.c_str());

	//cout << "(sn) path=" << path.toStdString() << endl;

	ifstream fin((const char *)path.toAscii());
	if (!fin) {
		fin.close();
		return location;
	}

	getline(fin, location);
	fin.close();

	//cout << "(sn) location=" << location << endl;
	return location;
}

/*
 * 2012-03-07 (s.noguchi) MICARY,Impulseファイルの保存とリストア
 */
void GUISeparationTFNew::saveFileLocation(string fname, string location) {
	QDir dir;
	QString path = QString("%1/%2/%3").arg(dir.homePath(), TEMPORARY_PATH.c_str(), fname.c_str());
	//cout << "(sn) path=" << path.toStdString() << endl;

	ofstream fout((const char *)path.toAscii());
	fout << location << endl;
	fout.close();
}
