#include "GUITSPListFile.h"
#include "ui_GUITSPListFile.h"

#include "GUICommon.h"
#include "UIException.h"
#include "GUITSPListFileTemplateNew.h"
#include "UIGraph.h"
#include "UIDocument.h"
#include "UIApp.h"


GUITSPListFile::GUITSPListFile(HT::TSP::UIDocument* doc, QWidget *parent) :
	QWidget(parent), ui(new Ui::GUITSPListFile) {
	ui->setupUi(this);
	if (doc) {
		this->doc = doc;
	} else {
		this->doc = new HT::TSP::UIDocument();
	}
}

GUITSPListFile::~GUITSPListFile() {
	IFDEBUG(DP("GUITSPListFile Destructor"));
	delete ui;
	delete doc;
}

void GUITSPListFile::templateNew() {

	try {
		GUITSPListFileTemplateNew dialog(this);
		connect(&dialog, SIGNAL(createdTemplate(QString)), this, SLOT(dispTemplate(QString)) );
		dialog.exec();
		disconnect(&dialog, SIGNAL(createdTemplate(QString)), this, SLOT(dispTemplate(QString)) );
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
	return;
}

void GUITSPListFile::openFile(QString file) {

	std::ifstream ifs(file.toStdString().c_str());
	if(ifs.fail())
	{
		THROW_WARNING(QString(tr("Can't open file:%1")).arg(file));
	}
	std::stringstream buf;
	buf << ifs.rdbuf();
	dispTemplate(buf.str().c_str());
	doc->setFileName(file.toStdString());
}

void GUITSPListFile::dispTemplate(QString text) {

	try {
		ui->text->setPlainText(text);
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUITSPListFile::dispGraph() {

	try {
		QString text = ui->text->toPlainText();
		// ファイルの取得
		harkio::TSPListFile file;
		file.readText(text.toStdString());

		// データの取得
		std::vector<harkio::UISrc<> > *src_list = &(file.src_list);
		std::vector<double> xyz(3);
		std::vector<std::vector<double> > lists; // 音源情報の配列

		for (uint i=0 ; i < src_list->size(); i++) {
			xyz[0] = (*src_list)[i].x;
			xyz[1] = (*src_list)[i].y;
			xyz[2] = (*src_list)[i].z;
			lists.push_back(xyz);
		}
		// プロット
		ScatterPlot<double> graph(&lists);
		graph.setTitle("Source Position");
		graph.setStyle(ScatterPlot<>::PlotStyle::LINESPOINTS);
		graph.plot(harktoolDir());

	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}
}

void GUITSPListFile::saveFile(bool dialog) {

	try {

		QString fname = doc->getFileName().c_str();
		bool saved=false;
		if (!fname.isEmpty()) {
			saved = true;
		}
		if (dialog || !saved) {
			fname = QFileDialog::getSaveFileName(this, "", doc->getLastPath(fname.toStdString()).c_str());
			if (fname.isEmpty()) {
				return;
			}
		}
		
		QFile file(fname);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
			THROW_WARNING(QString(tr("Can't open file:%1")).arg(fname));
			return;
		}
		doc->setFileName(fname.toStdString());
		HT::UIDocument::setLastPath(fname.toStdString());

		// 書き込み
		QTextStream out(&file);
		out << ui->text->toPlainText();
		
		if (!dialog && saved) {
			DISP_INFO(tr("File Overwritten", "上書き保存しました"));
		}
	} catch (std::exception& exp) {
		DISP_DIALOG(exp);
		return;
	}

}

QString GUITSPListFile::text() {
	return ui->text->toPlainText();
}

void GUITSPListFile::changeEvent(QEvent *e) {
	QWidget::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}
