#include <fstream>
#include "UIData.h"
#include <boost/format.hpp>
#include <boost/regex.hpp>
#include <boost/shared_ptr.hpp>
#include <QString>
#include <libharkio/textdata.h>
#include "common.h"
#include "UIDatData.h"
#include "UITFData.h"

using namespace std;
using namespace harkio;
using namespace boost;

namespace HT {

HT::UIDocument* UIData::getDocument() {
	return document;
}
void UIData::setDocument(UIDocument* document) {
	this->document = document;
}

void Transform::BasicData(HT::UIData& arg_data, int* mic, int* src, int* fft, int* fs) {

	if (dynamic_cast<HT::MUSIC::UIData*> (&arg_data)) {
		HT::MUSIC::UIData* data = (HT::MUSIC::UIData*)&arg_data;
		if (data->isHARKIO()) {
			FMT *fmt = data->getHARKIO()->getData()->getChunk<FMT> ();
			M2PG *m2pg = data->getHARKIO()->getData()->getChunk<M2PG> ();
			*mic = fmt->getNMic();
			*fft = fmt->getNFFT();
			*fs = fmt->getNFS();
			*src = m2pg->getND();
		} else {
			*mic = data->getNMic();
			*src = data->getNSrc();
			*fft = data->getNFFT();
			*fs = data->getNFS();
		}
	} else if (dynamic_cast<HT::GSS::UIData*> (&arg_data)) {
		HT::GSS::UIData* data = (HT::GSS::UIData*)&arg_data;
		if (data->isHARKIO()) {
			FMT *fmt = data->getHARKIO()->getData()->getChunk<FMT> ();
			int id = fmt->getFormatId();
			if (id == FMT::SM) {
				SM *sm = data->getHARKIO()->getData()->getChunk<SM> ();
				*src = sm->getNSrc();
			} else if (id == FMT::GTF) {
				GTF *gtf = data->getHARKIO()->getData()->getChunk<GTF> ();
				*src = gtf->getNSrc();
			}
			*mic = fmt->getNMic();
			*fft = fmt->getNFFT();
			*fs = fmt->getNFS();
		} else {
			*mic = data->getNMic();
			*src = data->getNSrc();
			*fft = data->getNFFT();
			*fs = data->getNFS();
		}
	}
	else {
		THROW_CRITICAL("Unknown data for transformation");
	}

}

TFMatrix<HT_COMPLEX_DATA_TYPE>& Transform::commonIFFTTFMatrix(HT::UIData& arg_data) {

	if (dynamic_cast<HT::MUSIC::UIData*>(&arg_data)) {
		HT::MUSIC::UIData* data = (HT::MUSIC::UIData*)&arg_data;
		int NFFT; int NMIC; int NSRC; int NFS;
		BasicData(*data, &NMIC, &NSRC, &NFFT, &NFS);

		if (data->isHARKIO()) {
			M2PG *m2pg = data->getHARKIO()->getData()->getChunk<M2PG>();
			delete old_matrix;
			old_matrix = data->IFFT(HARKIOTFToTFMatirx(m2pg->getTF()), NFFT, NMIC, NSRC);
			return *old_matrix;
		} else {
			delete old_matrix;
			old_matrix = data->IFFT(*data->getMatrix(), NFFT, NMIC, NSRC);
			return *old_matrix;
		}
	} else if (dynamic_cast<HT::GSS::UIData*>(&arg_data)) {
		HT::GSS::UIData* data = (HT::GSS::UIData*)&arg_data;
		int NFFT; int NMIC; int NSRC; int NFS;
		BasicData(*data, &NMIC, &NSRC, &NFFT, &NFS);

		if (data->isHARKIO()) {
			FMT *fmt =   data->getHARKIO()->getData()->getChunk<FMT>();
			int id = fmt->getFormatId();
			if (id == FMT::SM) {
				SM *sm =   data->getHARKIO()->getData()->getChunk<SM>();
				delete old_matrix;
				old_matrix = data->IFFT(HARKIOTFToTFMatirx(sm->getSM()), NFFT, NMIC, NSRC);
			} 
			else if (id == FMT::GTF) 
			{
				GTF *gtf =   data->getHARKIO()->getData()->getChunk<GTF>();
				delete old_matrix;
				//old_matrix = data->IFFT(HARKIOTFToTFMatirx(gtf->getTF()), NFFT, NMIC, NSRC);

				/* グラフ表示が反転するバグの対応 */
				TFMatrix<HT_COMPLEX_DATA_TYPE> matrix_tmp;
				matrix_tmp = HARKIOTFToTFMatirx(gtf->getTF());
				old_matrix  = data->IFFT(matrix_tmp.conj() , NFFT, NMIC, NSRC);
			}
			return *old_matrix;
		} else {
			delete old_matrix;
			old_matrix = data->IFFT(*data->getMatrix(), NFFT, NMIC, NSRC);
			return *old_matrix;
		}
	}
/*
	else if (dynamic_cast<HT::GSSMAT::UIData*>(&arg_data)) {
		HT::GSSMAT::UIData* data = (HT::GSSMAT::UIData*)&arg_data;
		if (data->isHARKIO()) {
			SM *sm =   data->getHARKIO()->getData()->getChunk<SM>();
			return HARKIOTFToTFMatirx(sm->getSM());
		} else {
			return *data->getMatrix();
		}
	}
*/
	 else {
		THROW_CRITICAL("Unknown data for transformation");
	}
}

TFMatrix<HT_COMPLEX_DATA_TYPE>& Transform::commonTFMatrix(HT::UIData& arg_data) {

	if (dynamic_cast<HT::MUSIC::UIData*>(&arg_data)) {
		HT::MUSIC::UIData* data = (HT::MUSIC::UIData*)&arg_data;
		if (data->isHARKIO()) {
			M2PG *m2pg = data->getHARKIO()->getData()->getChunk<M2PG>();
			return HARKIOTFToTFMatirx(m2pg->getTF());
		} else {
			return *data->getMatrix();
		}
	} else if (dynamic_cast<HT::GSS::UIData*>(&arg_data)) {
		HT::GSS::UIData* data = (HT::GSS::UIData*)&arg_data;
		if (data->isHARKIO()) {
			FMT *fmt =   data->getHARKIO()->getData()->getChunk<FMT>();
			int id = fmt->getFormatId();
			if (id == FMT::SM) {
				SM *sm = data->getHARKIO()->getData()->getChunk<SM>();
				return HARKIOTFToTFMatirx(sm->getSM());
			} else if (id == FMT::GTF) {
				GTF *gtf = data->getHARKIO()->getData()->getChunk<GTF>();
				return HARKIOTFToTFMatirx(gtf->getTF());
			}
		} else {
			return *data->getMatrix();
		}
	}
/*
	else if (dynamic_cast<HT::GSSMAT::UIData*>(&arg_data)) {
		HT::GSSMAT::UIData* data = (HT::GSSMAT::UIData*)&arg_data;
		if (data->isHARKIO()) {
			SM *sm =   data->getHARKIO()->getData()->getChunk<SM>();
			return HARKIOTFToTFMatirx(sm->getSM());
		} else {
			return *data->getMatrix();
		}
	}
*/
	 else {
		THROW_CRITICAL("Unknown data for transformation");
	}
}
// GTF::TF_LIST_TYPE(HARKIO伝達関数) -> TFMatrix(旧形式)
TFMatrix<HT_COMPLEX_DATA_TYPE>& Transform::HARKIOTFToTFMatirx(harkio::GTF::TF_LIST_TYPE* tf) {

	// tf[ns][nmic][nf]
	uint NSrc = (*tf).size();
	uint NMic = (*tf)[0].size();
	uint NFFTplus1 = (*tf)[0][0].size();

	delete old_matrix;
	old_matrix = new TFMatrix<HT_COMPLEX_DATA_TYPE> (3, NFFTplus1, NMic, NSrc);

	for (uint src = 0; src < NSrc; src++) { // ライブラリTFデータをコピーする
		for (uint mic = 0; mic < NMic; mic++) {
			for (uint fft = 0; fft < NFFTplus1; fft++) {
				(*old_matrix).matrix[fft][mic][src].real() = (*tf)[src][mic][fft].real();
				(*old_matrix).matrix[fft][mic][src].imag() = (*tf)[src][mic][fft].imag();
			}
		}
	}
	return *old_matrix;
}

// M2PG::TF_LIST_TYPE(HARKIO伝達関数) -> TFMatrix(旧形式)
TFMatrix<HT_COMPLEX_DATA_TYPE>& Transform::HARKIOTFToTFMatirx(harkio::M2PG::TF_LIST_TYPE* tf) {
	// tf[nd][nr][nmic][nfft]
	uint NSrc = (*tf).size();
	uint NMic = (*tf)[0][0].size();
	uint NFFTplus1 = (*tf)[0][0][0].size();

	delete old_matrix;
	old_matrix = new TFMatrix<HT_COMPLEX_DATA_TYPE> (3, NFFTplus1, NMic, NSrc);

	for (uint src = 0; src < NSrc; src++) { // ライブラリTFデータをコピーする
		for (uint mic = 0; mic < NMic; mic++) {
			for (uint fft = 0; fft < NFFTplus1; fft++) {
				(*old_matrix).matrix[fft][mic][src].real() = (*tf)[src][0][mic][fft].real();
				(*old_matrix).matrix[fft][mic][src].imag() = (*tf)[src][0][mic][fft].imag();
			}
		}
	}
	return *old_matrix;
}

// vector<UISrc<T> >(旧形式マイク) -> std::vector<std::vector<T> > (GNUPlot散布図)
std::vector<std::vector<double> >& Transform::OldMicToGNUPlotVector(std::vector<HT::UIMic<HT_DATA_TYPE> >* mic) {

	mics.resize(0);
	mics.resize(mic->size());
	for (uint i = 0; i < mic->size(); i++) {
		mics[i].push_back((*mic)[i].x);
		mics[i].push_back((*mic)[i].y);
		mics[i].push_back((*mic)[i].z);
	}
	return mics;
}
// vector<UISrc<T> >(旧形式ソース) -> std::vector<std::vector<T> > (GNUPlot散布図)
std::vector<std::vector<double> >& Transform::OldSrcToGNUPlotVector(std::vector<HT::UISrc<HT_DATA_TYPE> >* src) {

	srcs.resize(0);
	srcs.resize(src->size());
	for (uint i = 0; i < src->size(); i++) {
		srcs[i].push_back((*src)[i].x);
		srcs[i].push_back((*src)[i].y);
		srcs[i].push_back((*src)[i].z);
	}
	return srcs;
}

// M2PG::SRC_LIST_TYPE(HARKIOマイク) -> std::vector<std::vector<T> > (GNUPlot散布図)
std::vector<std::vector<double> >& Transform::HARKIOMicToGNUPlotVector(harkio::FMT::MIC_LIST_TYPE* mic) {

	mics.resize(0);
	mics.resize(mic->size());
	for (uint i = 0; i < mic->size(); i++) {
		mics[i].push_back((*mic)[	i].x);
		mics[i].push_back((*mic)[i].y);
		mics[i].push_back((*mic)[i].z);
	}
	return mics;
}
// FMT::MIC_LIST_TYPE(HARKIOソース) -> std::vector<std::vector<T> > (GNUPlot散布図)
std::vector<std::vector<double> >& Transform::HARKIOSrcToGNUPlotVector(harkio::M2PG::SRC_LIST_TYPE* src) {

	srcs.resize(0);
	srcs.resize(src->size());
	for (uint i = 0; i < src->size(); i++) {
		srcs[i].push_back((*src)[i][0].x);
		srcs[i].push_back((*src)[i][0].y);
		srcs[i].push_back((*src)[i][0].z);
	}
	return srcs;
}

} //namespace
