#include "UIGraph.h"
#include "common.h"
#include <complex>
#include <math.h>
#include <QString>

using namespace std;

template <> double calcZValue<std::complex<double> >(int z_unit, std::complex<double> value) {

	double re = value.real();
	double im = value.imag();
	switch(z_unit) {
		case GNUPlot<>::ZUnit::REAL:
			return re;
			break;
		case GNUPlot<>::ZUnit::AMPLITUDE:
		return sqrt(re*re + im*im);
			break;
		case GNUPlot<>::ZUnit::DB:
			return log10(sqrt(re*re + im*im)) * 10;
			break;
		case GNUPlot<>::ZUnit::PHASE:
			return atan2(im, re) * 180/M_PI;
			break;
		default:
			THROW_CRITICAL(QString("Unknown Z axis's unit:%1").arg(z_unit));
		break;
	}
}

template <> std::string toString<std::complex<double> >(std::complex<double> value, bool complex) {
	double re = value.real();
	double im = value.imag();

	std::string str;
	if (complex) {
		str = (boost::format("%f%+fi") % re  % im ).str();
	} else {
		str = (boost::format("%f") % re ).str();
	}
	return str;
}
