#ifndef UITFDOCUMENT_H
#define UITFDOCUMENT_H

#include <iostream>
#include <string>
#include <sstream>
#include <map>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <vector>
#include <utility>

#include "UIDocument.h"
#include "UITFData.h"
#include "UIGraph.h"
#include "common.h"

//! GSS の伝達関数を読み書きする時に使用する構造体
typedef struct{
	int lFs ;		///<\~Japanese サンプリング周波数[Hz]
	int lNFFT ;	///<\~Japanese FFT長[point]
	int lNMic ;	///<\~Japanese マイクロホン位置数
	int lNSrc ;	///<\~Japanese 音源位置数
	int lNFreq ;	///<\~Japanese 周波数ライン数[line]
	int lMeshFlg ;	///<\~Japanese 音源位置が格子状になっているかのflag
	int lMeshX ;	///<\~Japanese X方向の格子数
	int lMeshY ;	///<\~Japanese Y方向の格子数
	int lMeshZ ;	///<\~Japanese Z方向の格子数
	int iVersion;	///<\~Japanese バージョン番号
	int iFileType;	///<\~Japanese ファイルタイプ（0:TF_CONJ, 1:分離行列）
	char cPad[92] ;	///<\~Japanese パッド
} NAE_TFDBFileHeader ;


namespace HT {
class UIGraph;

namespace GSS {
class UIData;

//! GSSのTFのドキュメントクラス
class UIDocument : public HT::UIDocument{
private:
	std::string srcin_filename;	// srcinfファイル名
	std::string mic_filename;	// micファイル名
	boost::shared_ptr<HT::GSS::UIData> data;

public:
	UIDocument(std::string name, std::string filename);
	UIDocument();
	virtual ~UIDocument();

	void setSrcInfFile(std::string name) {
		srcin_filename = name;
	}
	std::string& getSrcInfFile() {
		return srcin_filename;
	}

	void setMicFile(std::string name) {
		mic_filename = name;
	}
	std::string& getMicFile() {
		return mic_filename;
	}

	//! このドキュメントに対して、Dataインスタンスの生成と包含を同時に行う
	UIData *newTFData(UIDocument *doc);

	HT::GSS::UIData * getUIData(bool exception=true);
	HT::UIData* getBaseUIData();

	void mkmusictf(std::string mic_fname, int fs, int fft, int degree_interval, int nir);
	//! ドキュメントに、ファイルを読み込む
	void loadFile(std::string filename);
	//!  harkio でファイルを読み込む
	void loadFileByLib(std::string filename);
	//! 　harkioを使わないでファイルを読み込む
	void loadFileNormally(std::string filename);
};

} //namespace GSS
} //namespace
#endif
