#ifndef UIVERIFY_H
#define UIVERIFY_H

#include <iostream>
#include <gtk/gtk.h>
#include <boost/lexical_cast.hpp>

#include "UITFDocument.h"
#include "UITFData.h"
#include "UIDatDocument.h"
#include "UIMicDocument.h"
#include "common.h"

namespace HT {

namespace MUSIC {
	class UIDocument;
	class UIData;
} //namespace MUSIC

namespace GSS {
	class UIDocument;
	class UIData;

} //namespace GSS

namespace MIC {
	class UIDocument;
} //namespace MIC


//! Verify メニュークラス
/*!
 * GSS,MUSICの伝達関数のVerifyを行うクラス
 * 対象となる、グループ(GSS,MUSIC等)とタブを指定してから、Verifyを実行する
 */
class UIVerify {
protected:
	HT::UIApp::GROUPID group;	//! 対象グループ
	std::string tab1;			//! 比較ドキュメント１
	std::string tab2;			//! 比較ドキュメント２
	HT::UIData *tab1_data;
	HT::UIData *tab2_data;
	std::vector<UIGraph *> graph_list;	//! 差分のグラフの配列(差分と割合一緒)
public:
	enum TARGET {DIFF, DIFF_RATIO};
	int total_count;	//! 対象となる数値の個数(複素数なので、マトリックスの２倍である)
	int tab1_error;		//! tab1の無限大等のエラー個数
	int tab2_error;		//! tab1の無限大等のエラー個数
	CheckVector<double> *diff_info;			//! 差分の統計情報保持
	CheckVector<double> *diff_ratio_info;	//! 差分率の統計情報保持

	UIVerify() {
		group = HT::UIApp::GSS;
		diff_info = NULL;
		diff_ratio_info = NULL;
		tab1_data=NULL;
		tab2_data=NULL;
	}
	virtual ~UIVerify() {
		// グラフ削除
		std::vector<UIGraph *>::iterator it = graph_list.begin();
		while(it != graph_list.end()) {
			delete(*it);
			it++;
		}
	};
	//! グループ設定関数
	//\{
	//! セット
	void setGroup(HT::UIApp::GROUPID id) {
		// FAIL SAFE Verifyに関係するグループは、２つだけ
		if(id != HT::UIApp::GSS && id != HT::UIApp::MUSIC ) {
			group = HT::UIApp::GSS;
		} else {
			group = id;
		}
	}
	//! 取得
	HT::UIApp::GROUPID getGroup() {
		return group;
	}
	//\}

	//! タブ設定関数
	//\{
	//! セット
	void setTab(std::string var1, std::string var2) {
		tab1 = var1;
		tab2 = var2;
	}
	void getTab(std::string *tab1, std::string *tab2) {
		*tab1 = this->tab1;
		*tab2 = this->tab2;
	}
	//\}
	
	//! 比較するデータのセット
	void setData(UIData *tab1, UIData *tab2) {
		tab1_data = tab1;
		tab2_data = tab2;
	}

	//! Verifyの実行
	//\{
	void verifyGSS();
	void verifyMUSIC();
	//\}

	//! 差分のグラフを作成する
	void createGraphDiff(int mic, HT::UIVerify::TARGET target, HT::ZAxis::ZAXIS axis); // マイク判定後処理
};

} //namespace
#endif
